<?php
/**
 * GENERIC Config
 * Contains the generic configuration class - so no need to be in here
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
 * @copyright GPL 2
 *
 */
namespace leau\co\wp_favicons;

if (!class_exists("Config"))
{
	/**
	 * Holds the configuration of the plugin
	 * Contains only simple vars and helpers no stuff that influences the action
 	 * @package WP-Favicons
 	 * @since 0.4.0
 	 * @author Edward de Leau <e@leau.net>, {@link http://edward.de.leau.net}
  	 * @copyright GPL 2
	 */
	class Config
	{
		private $_options_name;
		private $_options_array = array();
		private $_settings_group;
		private $_page_title;
		private $_menu_title;
		private $_capability;
		private $_menu_slug;
		private $_plugin_slug;
		private $_modules = array();
		private $_mainpage;
		private $_module_plugin = array();
		private $_module_plugin_register = array();
		private $_menu_icon;
		private $_plugin_version;

		// singleton instance
		private static $instance;

		// private constructor function
		// to prevent external instantiation
		final private function __construct()
		{

		}
		private function __clone()
		{
		}

		// getInstance method
		public static function getInstance()
		{
			if(!self::$instance)
			{
				self::$instance = new self();
			}
			return self::$instance;
		}

		/* ------------ options name ------------------ */

		public function GetOptionsName()
		{
			$instance = self::getInstance();
			return $instance->_options_name;
		}

		public function SetOptionsName($optionsName)
		{
			$instance = self::getInstance();
			$instance->_options_name = $optionsName;
		}

		/* ------------ options array ------------------ */

		/**
		 *
		 * Enter description here ...
		 */
		public function GetOptionsAsArray()
		{
			$instance = self::getInstance();
			return $instance->_options_array;
		}

		public function GetOptionsArrayValue($key)
		{
			$instance = self::getInstance();
			if (array_key_exists($key, $instance->_options_array)) {
				return $instance->_options_array[$key];
			}
			else
			{
				return false;
			}
		}

		/**
		 * @todo 0.5.0 check this one
		 * Enter description here ...
		 * @param unknown_type $optionsArray
		 */
		public function SetOptionsFromArray($optionsArray)
		{
			$instance = self::getInstance();
			$instance->_options_array = $optionsArray;
		}

		/* ------------ Settings Group ------------------ */

		/**
		 *
		 * Enter description here ...
		 */
		public function GetSettingsGroupName()
		{
			$instance = self::getInstance();
			return $instance->_settings_group;
		}

		/**
		 *
		 * Enter description here ...
		 * @param unknown_type $settingsGroup
		 */
		public function SetSettingsGroupName($settingsGroup)
		{
			$instance = self::getInstance();
			$instance->_settings_group = $settingsGroup;
		}

		/* ------------ Plugin Version ------------------ */

		public function GetPluginVersion()
		{
			$instance = self::getInstance();
			return $instance->_plugin_version;
		}

		public function SetPluginVersion($version)
		{
			$instance = self::getInstance();
			$instance->_plugin_version = $version;
		}

		/* ------------ Page Title ------------------ */

		public function GetPageTitle()
		{
			$instance = self::getInstance();
			return $instance->_page_title;
		}

		public function SetPageTitle($pageTitle)
		{
			$instance = self::getInstance();
			$instance->_page_title = $pageTitle;
		}

		/* ------------ Menu Title ------------------ */

		public function GetMenuTitle()
		{
			$instance = self::getInstance();
			return $instance->_menu_title;
		}

		public function SetMenuTitle($menuTitle)
		{
			$instance = self::getInstance();
			$instance->_menu_title = $menuTitle;
		}

		/* ------------ Capability ------------------ */

		public function GetCapability()
		{
			$instance = self::getInstance();
			return $instance->_capability;
		}

		public function SetCapability($capability)
		{
			$instance = self::getInstance();
			$instance->_capability = $capability;
		}

		/* ------------ Menu Slug ------------------ */

		public function GetMenuSlug()
		{
			$instance = self::getInstance();
			return $instance->_menu_slug;
		}

		public function SetMenuSlug($menuSlug)
		{
			$instance = self::getInstance();
			$instance->_menu_slug = $menuSlug;
		}

		/* ------------ Plugin Slug ------------------ */

		public function GetPluginSlug()
		{
			$instance = self::getInstance();
			return $instance->_plugin_slug;
		}

		public function SetPluginSlug($pluginSlug)
		{
			$instance = self::getInstance();
			$instance->_plugin_slug = $pluginSlug;
		}

		/* ------------ Modules ------------------ */

		/**
		 * Get one specific module
		 * Get a specific module by number
		 * @param unknown_type $n
		 */
		public function GetModuleByOrder($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n];
		}

		/**
		 * Get a page title of a menu based on the id
		 * @param $n
		 */
		public function GetModulePageTitle($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n]['page_title'];
		}

		/**
		 * Get a menu title of a menu based on the id
		 * @param $n
		 */
		public function GetModuleMenuTitle($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n]['menu_title'];
		}

		/**
		 * Get a menu slug of a menu based on the id
		 * @param $n
		 */
		public function GetModuleMenuSlug($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n]['menu_slug'];
		}

		/**
		 * Get a section header title of a menu based on the id
		 * @param $n
		 */
		public function GetModuleSectionHeaderTitle($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n]['section_header_title'];
		}

		/**
		 * Get a section header text of a menu based on the id
		 * @param $n
		 */
		public function GetModuleSectionHeaderText($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n]['section_header_text'];
		}

		/**
		 * Get section help text of a menu based on the id
		 * @param $n
		 */
		public function GetModuleHelpText($n)
		{
			$instance = self::getInstance();
			return $instance->_modules[$n]['help_text'];
		}

		/**
		 * Gets all modules in a hierarchical array
		 * Enter description here ...
		 */
		public function GetAllModules()
		{
			$instance = self::getInstance();
			return $instance->_modules;
		}

		/**
		 * Add a new module
		 * Adds a new module by passing the module configuration data in array
		 * @param array $module
		 */
		public function AddModule($module)
		{
			$instance = self::getInstance();
			if (is_array($module))
			{
				$instance->_modules[] = $module;
			}
			else
			{
				throw new Exception('Invalid Parameter: should be array');
			}
		}

		public function AddMenuIcon($icon)
		{
			$instance = self::getInstance();
			$instance->_menu_icon = $icon;
		}

		public function GetMenuIcon()
		{
			$instance = self::getInstance();
			return $instance->_menu_icon;
		}

		/* ------------ Main page ------------------ */

		/**
		 *
		 * Enter description here ...
		 * @param unknown_type $mainPage
		 */
		public function SetMainPage($mainPage)
		{
			$instance = self::getInstance();
			$instance->_mainpage = $mainPage;
		}

		/**
		 *
		 * Enter description here ...
		 * @param unknown_type $mainPage
		 */
		public function GetMainPage()
		{
			$instance = self::getInstance();
			return $instance->_mainpage;
		}

		/* ------------ Module Plugins ------------------ */

		/**
		 *
		 * Enter description here ...
		 * @param unknown_type $moduleID
		 * @param unknown_type $pluginID
		 * @param unknown_type $pluginVar
		 * @param unknown_type $pluginValue
		 */
		public function SetModulePlugin($moduleID,$pluginID,$pluginVar,$pluginValue)
		{
			$instance = self::getInstance();
			$instance->_module_plugin[$moduleID][$pluginID][$pluginVar] = $pluginValue;
		}

		public function GetModulePluginPageName($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			return $instance->GetPluginSlug() . $moduleID . '_page';
		}

		public function GetModulePluginSectionName($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			return $instance->GetPluginSlug() . $moduleID . '_' . $pluginID . '_section';
		}

		public function GetModulePluginVar($moduleID,$pluginID,$pluginVar)
		{
			$instance = self::getInstance();
			return $instance->_module_plugin[$moduleID][$pluginID][$pluginVar];
		}

		public function GetModulePluginTitle($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			return $instance->_module_plugin[$moduleID][$pluginID]['title'];
		}

		public function GetModulePluginHeader($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			return $instance->_module_plugin[$moduleID][$pluginID]['header'];
		}



		/* ------------ Module Plugin Fields ------------------ */

		/**
		 * @param string @moduleID
		 * @param string @pluginID
		 * @param int @pluginField id of pluginField
		 * @param array @pluginValue an array of the values of the field
		 */
		public function SetModulePluginField($moduleID,$pluginID,$pluginField,$pluginValue)
		{
			$instance = self::getInstance();
			$instance->_module_plugin[$moduleID][$pluginID]['fields'][$pluginField] = $pluginValue;
		}

		public function GetModulePluginFields($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			return $instance->_module_plugin[$moduleID][$pluginID]['fields'];
		}

		public function CheckExistenceOfFields($moduleID, $pluginID)
		{
			$instance = self::getInstance();
			$current_fields = $instance->GetModulePluginFields($moduleID,$pluginID);
			$fields_to_add = array();
			foreach ($current_fields as $field)
			{
				$fields_to_add[$field['name']] = $field['default'];
			}
			$instance->AddOptions($fields_to_add);
		}

		public function HasFields($moduleID, $pluginID)
		{
			$instance = self::getInstance();
			$current_fields = $instance->GetModulePluginFields($moduleID,$pluginID);
			if (is_array($current_fields)) {return true;} else {return false;}
		}

		public function GetFieldValue($moduleID, $pluginID, $fieldID, $fieldKey)
		{
			$instance = self::getInstance();
			return $instance->GetOptionsArrayValue($instance->_module_plugin[$moduleID][$pluginID]['fields'][$fieldID][$fieldKey]);
		}

		/**
		 * @todo probably this should be done in a better way it means that
		 * the first boolean of a plugin should be true, so that always the
		 * first option MUST be if it should be turned on or off.
		 * Enter description here ...
		 * @param unknown_type $moduleID
		 * @param unknown_type $pluginID
		 */
		public function IsPluginActive($moduleID, $pluginID)
		{
			$instance = self::getInstance();
			if ($instance->HasFields($moduleID, $pluginID))
			{
				if ($instance->GetFieldValue($moduleID, $pluginID, 0, 'name') == '1')
            	{
            		return true;
            	}
            	else
            	{
            		return false;
            	}
			}
		}

		/* ------------ Module Plugin Register ------------------ */

		public function RegisterModulePlugin($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			$instance->_module_plugin_register[$moduleID][$pluginID] = true;
		}

		public function UnRegisterModulePlugin($moduleID,$pluginID)
		{
			$instance = self::getInstance();
			$instance->_module_plugin_register[$moduleID][$pluginID] = false;
		}

		/**
		 * Returns all or one specific module's plugins
		 * @param string $module the module requested or empty
		 * @return array the list of plugins
		 */
		public function GetRegisteredPlugins($module='')
		{
			$instance = self::getInstance();
			if ($module=='')
			{
				return $instance->_module_plugin_register;
			}
			else
			{
				return $instance->_module_plugin_register[$module];
			}
		}

		/**
		 * Update Options
		 * Helper function to update all the options for the plugin note that
		 * add_option almost always does nothing because we have an array
		 * not a single field
		 * @uses update_option {@link http://codex.wordpress.org/Function_Reference/update_option}
		 */
		function AddOptions($new_values)
		{
			$instance = self::getInstance();
			$values_to_add = array();
			if (!empty($new_values))
			{
				foreach ($new_values as $key => $value)
				{
					if (!array_key_exists($key,$instance->_options_array))
					{
						$values_to_add[$key] = $value;
					}
				}
			}
			$instance->UpdateOptions($values_to_add);
		}

		/**
		 * Updates the options array
		 * @param $new_values
		 */
		function UpdateOptions($new_values)
		{
			$instance = self::getInstance();
			if (!empty($new_values))
			{
				foreach ($new_values as $key => $value)
				{
					$instance->_options_array[$key] = $value;
				}
				update_option($instance->_options_name, $instance->_options_array);
			}
		}


	}
}
