<?php
/**
 * New file yet to be documented
 * Enter description here ...
 * @author cogmios
 *
 */
namespace leau\co\wp_favicons;

if (!class_exists("FaviconFactory"))
{
	class FaviconFactory
	{
		private $_favicon;
		private $_active_filter;

		/**
		 *
		 * Enter description here ...
		 * @param $favicon
		 * @param $active_filter
		 */
		function __construct($favicon, $active_filter)
		{
			$this->_favicon = $favicon;
			$this->_active_filter = $active_filter;

			if (Config::GetOptionsArrayValue(Config::GetPluginSlug() . 'use_cache'))
			{
				$this->SetAdvancedFavicon();
			}
			else
			{
				$this->SetSimpleFavicon();
			}
		}

		/**
		 *
		 * Enter description here ...
		 */
		function SetSimpleFavicon()
		{
			$this->_favicon->SetPublicFaviconUri(
				'http://www.google.com/s2/favicons?domain=' .
					parse_url($this->_favicon->GetUri(), PHP_URL_HOST));
		}

		/**
		 *
		 * Enter description here ...
		 */
		function SetAdvancedFavicon()
		{
			$this->_favicon->ReadFromDatabase();
			if (!$this->_favicon->ExistsInDatabase()) {
				/* check time buffer first */
				apply_filters( Config::GetPluginSlug() . 'timebuffer', $this->_favicon);
				if ($this->_favicon->GetState()) {
					//Log::F($this->_favicon,' [ START ]------------------------------------------ ');
					Log::F($this->_favicon,'Does not exist yet in database');
					$this->_favicon->SetNewInDb();
					$this->FaviconLoop();

					if ($this->_favicon->FoundStatus())
					{
						Log::F($this->_favicon,'Write in database after icon loop:');
						$this->_favicon->WriteInDatabase();
						if (!$this->_favicon->ExistsInDatabase()) {
							Log::F($this->_favicon,'Still does not exist yet in database:');
							$this->_favicon->SetPublicFaviconUri('');
						}
					}
					else
					{
						/* no icon was found and no default set
						 * @todo add extra database status
						 */
						$this->SetSimpleFavicon();
						return;
					}
				}
				else
				{
					$this->SetSimpleFavicon();
					return;
				}
			}
			else
			{
				//Log::F($this->_favicon,' [ START ]------------------------------------------ ');
			}

			if ($this->_favicon->GetFilters() == 'NOCACHE')
			{
				Log::F($this->_favicon,$this->_favicon->GetFilters());
				Log::F($this->_favicon,'NON CACHED Version: not saved in disk cache');
				$this->_favicon->SetPublicFaviconUri($this->_favicon->GetFaviconUri());
			}
			else
			{
				$favicon_cache = new EdlLinkFaviconCache(
					$this->_favicon->GetUri(), 'favicon.' .
					$this->_favicon->GetType());

				if ($this->_favicon->GetNewInDb())
				{
					Log::F($this->_favicon,'New in database so disk caching');
					if ($data = $this->_favicon->GetData())
					{
						if ($favicon_cache->FileExistInCache())
						{
							Log::F($this->_favicon,'Icon already exists in disk cache, lets overwrite!');
						}

						$favicon_cache->Savedata($data);

						if (!$favicon_cache->FileExistInCache())
						{
							Log::F($this->_favicon,'Error could not be saved in disk cache');
							$this->_favicon->SetPublicFaviconUri('');
						}
						else
						{
							Log::F($this->_favicon,'Saved in disk cache:' . $this->_favicon->GetUri());
							$this->_favicon->SetPublicFaviconUri($favicon_cache->GetRelativeLocation()
								. 'favicon.' . $this->_favicon->GetType());
						}
					}
					else
					{
						Log::F($this->_favicon,'Error HAS NO DATA before save in disk cache');
						$this->_favicon->SetPublicFaviconUri('');
					}
				}
				else
				{
					/* get from cache - assume it is there to save speed */
					$this->_favicon->SetPublicFaviconUri($favicon_cache->GetRelativeLocation()
								. 'favicon.' . $this->_favicon->GetType());
				}
			}
		}

		/**
		 *
		 * Enter description here ...
		 */
		function GetAsHtml()
		{
			if ($this->_favicon->GetPublicFaviconUri() == '' )
			{
				return '';
			}
			else
			{
				return '<img '
					. 'src="' . $this->_favicon->GetPublicFaviconUri() . '" '
					. 'alt="' . Config::GetPluginSlug() . '" '
					. 'class="' . Config::GetPluginSlug() . $this->_active_filter . '" '
					. 'style="position: relative; top: 0; left: 0;" '
					. '/>';
			}
		}

		/**
		 * @todo second caller gets empty object....
		 * Enter description here ...
		 */
		function FaviconLoop()
		{
			Log::F($this->_favicon,'Start IconLoop With:' . $this->_favicon->GetUri());
			apply_filters( Config::GetPluginSlug() . 'search', $this->_favicon);
			apply_filters( Config::GetPluginSlug() . 'default', $this->_favicon);
			apply_filters( Config::GetPluginSlug() . 'convert', $this->_favicon);
			apply_filters( Config::GetPluginSlug() . 'default_non_cached', $this->_favicon);
		}

	}
}