<?php
/**
 * Widget: Shows the latest added icons
 * After some time this will correspond with the latest links added on your site
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
 * @copyright GPL 2
 */
namespace leau\co\wp_favicons;

if (!class_exists("widgets_icons_latest"))
{
	/**
 	 * Widget: Shows the latest added icons
 	 * @since 0.4.0
     * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 */

	class widgets_icons_latest extends \WP_Widget {

		/**
		 * Constructor
		 */
	 	function __construct()
    	{
    		$widget_ops = array('classname' => 'widget_icons_latest', 'description'
    			=> __('Shows your latest ♥ WP-Favicons which corresponds (after some time caching) '.
    			'with the latest outgoing links you published on your website. You can style '.
    			'the icons by using the stylesheet in the text widget settings under the ♥ WP-favicons '.
    			'Context settings.'));
    		$control_ops = array('width' => 200, 'height' => 350);
        	parent::__construct('WPFaviconsLatest', __('♥ Show Latest WP-Favicons'), $widget_ops, $control_ops);
    	}

		/**
		 * (non-PHPdoc)
		 * @see WP_Widget::widget()
		 */
		function widget($args, $instance) {
			extract( $args );
			echo $before_widget;
			$title = apply_filters( 'widget_title', empty($instance['title']) ? '' : $instance['title'], $instance, $this->id_base);
			$amount = empty($instance['amount']) ? 25 : $instance['amount'];
			$instance['text'] = $this->specificOutput($amount, false);
			$text = apply_filters( 'widget_text', $instance['text'], $instance );
			if ( !empty( $title ) ) { echo $before_title . $title . $after_title; } ?>
			<div class="widget_icons_latest"><?php echo $text; ?></div>
			<div class="widget_icons_latest">by <a href="http://wordpress.org/extend/plugins/wp-favicons/">WP-Favicons</a></div>
			<?php
			echo $after_widget;
		}

		/**
		 * (non-PHPdoc)
		 * @see WP_Widget::update()
		 */
		function update($new_instance, $old_instance)
		{
			$instance = $old_instance;
			$instance['title'] = empty($new_instance['title']) ? 'Latest Sites mentioned' : apply_filters('widget_title', $new_instance['title']);
			$instance['amount'] =  $new_instance['amount'];
			return $instance;
		}

		/**
		 * (non-PHPdoc)
		 * @see WP_Widget::form()
		 */
		function form($instance)
		{
		$instance = wp_parse_args( (array) $instance, array( 'title' => '', 'amount' => '' ) );
		$title = strip_tags($instance['title']);
		$amount = esc_textarea($instance['amount']);
		?>
		<p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
		<input 	class="widefat"
				id="<?php echo $this->get_field_id('title'); ?>"
				name="<?php echo $this->get_field_name('title'); ?>"
				type="text"
				value="<?php echo esc_attr($title); ?>" /></p>

		<p><label for="<?php echo $this->get_field_id('amount'); ?>"><?php _e('Amount:'); ?></label>
		<input 	type="text"
				class="widefat"
				id="<?php echo $this->get_field_id('amount'); ?>"
				name="<?php echo $this->get_field_name('amount'); ?>"
				value="<?php echo esc_attr($amount); ?>" /></p>
		<?php
		}

		/**
		 * The specific output for this widget
		 * Enter description here ...
		 */
		function specificOutput($amount='25', $defaults=true, $dofollow=true)
		{
			$output='';
			$result = Database::getLatestIcons($amount, $defaults);
			if ($result)
			{
				foreach($result as $key => $value)
				{
					$output.= '<a href="' . $value['uri'] . '">&nbsp;</a> ';
				}
			}
			return $output;
		}

	}
}
add_action('widgets_init',function(){
     return register_widget('leau\co\wp_favicons\widgets_icons_latest');
});